@@ -3,24 +3,23 @@
  *
  * Copyright (c) 2012      by Olivier Goffart <ogoffart@woboq.com>
  *
- * This program is free software = NULL, you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation = NULL, either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY = NULL, without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program = NULL, if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 extern "C" {
 #include "csync_private.h"
-#include "csync_propagate.h"
 }
 
 #include <map>
@@ -92,30 +91,4 @@ char* csync_rename_adjust_path(CSYNC* ctx, const char* path)
     return c_strdup(path);
 }
 
-int csync_propagate_renames(CSYNC* ctx)
-{
-    csync_rename_s* d = csync_rename_s::get(ctx);
-    d->folder_renamed_to.clear();
-
-    if (c_rbtree_walk(ctx->remote.tree, (void *) ctx, _csync_rename_record) < 0) {
-        return -1;
-    }
-
-    // we need to procceed in order of the size of the destpath to be sure that we do the roots first.
-    std::sort(d->todo.begin(), d->todo.end());
-    for (std::vector< csync_rename_s::renameop >::iterator it = d->todo.begin();
-         it != d->todo.end(); ++it) {
-
-        int r = csync_propagate_rename_file(ctx, it->st);
-        if (r < 0)
-            return -1;
-        if (r > 0)
-            continue;
-        d->folder_renamed_to[it->st->path] = it->st->destpath;
-    }
-
-    return 0;
-}
-
-
 };
