@@ -1,21 +1,22 @@
 /*
  * libcsync -- a library to sync a directory with another
  *
- * Copyright (c) 2008      by Andreas Schneider <mail@cynapses.org>
+ * Copyright (c) 2008-2013 by Andreas Schneider <asn@cryptomilk.org>
+ * Copyright (c) 2012-2013 by Klaas Freitag <freitag@owncloud.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -91,33 +92,37 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
 
   if ((file == NULL) || (fs == NULL)) {
     errno = EINVAL;
+    ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
     return -1;
   }
 
   path = file;
   switch (ctx->current) {
-  case LOCAL_REPLICA:
-    if (strlen(path) <= strlen(ctx->local.uri)) {
+    case LOCAL_REPLICA:
+      if (strlen(path) <= strlen(ctx->local.uri)) {
+        ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
+        return -1;
+      }
+      path += strlen(ctx->local.uri) + 1;
+      break;
+    case REMOTE_REPLICA:
+      if (strlen(path) <= strlen(ctx->remote.uri)) {
+        ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
+        return -1;
+      }
+      path += strlen(ctx->remote.uri) + 1;
+      break;
+    default:
+      path = NULL;
+      ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
       return -1;
-    }
-    path += strlen(ctx->local.uri) + 1;
-    break;
-  case REMOTE_REPLICA:
-    if (strlen(path) <= strlen(ctx->remote.uri)) {
-      return -1;
-    }
-    path += strlen(ctx->remote.uri) + 1;
-    break;
-  default:
-    path = NULL;
-    return -1;
-    break;
   }
 
   len = strlen(path);
 
   /* Check if file is excluded */
   excluded = csync_excluded(ctx, path,type);
+
   if (excluded != CSYNC_NOT_EXCLUDED) {
     CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "%s excluded  (%d)", path, excluded);
     if (excluded == CSYNC_FILE_EXCLUDE_AND_REMOVE) {
@@ -133,6 +138,9 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
       }
       return 0;
     }
+    if (excluded == CSYNC_FILE_SILENTLY_EXCLUDED) {
+        return 0;
+    }
   }
 
   h = _hash_of_file(ctx, file );
@@ -143,14 +151,15 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
 
   st = c_malloc(size);
   if (st == NULL) {
+    ctx->status_code = CSYNC_STATUS_MEMORY_ERROR;
     return -1;
   }
-  CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "==> file: %s - hash %llu, mtime: %llu",
-      path, (unsigned long long ) h, (unsigned long long) fs->mtime);
+  CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "==> file: %s - hash %llu, mtime: %llu, fileId: %s",
+      path, (unsigned long long ) h, (unsigned long long) fs->mtime, fs->file_id);
 
   /* Set instruction by default to none */
   st->instruction = CSYNC_INSTRUCTION_NONE;
-  st->md5 = NULL;
+  st->etag = NULL;
   st->child_modified = 0;
 
   /* check hardlink count */
@@ -161,7 +170,7 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
     }
 
     if (fs->mtime == 0) {
-      tmp = csync_statedb_get_stat_by_hash(ctx, h);
+      tmp = csync_statedb_get_stat_by_hash(ctx->statedb.db, h);
       CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "file: %s - mtime is zero!", path);
       if (tmp == NULL) {
         CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "file: %s - not found in db, IGNORE!", path);
@@ -184,7 +193,10 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
     goto out;
   }
   if (excluded > CSYNC_NOT_EXCLUDED || type == CSYNC_FTW_TYPE_SLINK) {
-    st->instruction = CSYNC_INSTRUCTION_IGNORE;
+      if( type == CSYNC_FTW_TYPE_SLINK ) {
+          st->error_status = CSYNC_STATUS_INDIVIDUAL_IS_SYMLINK; /* Symbolic links are ignored. */
+      }
+      st->instruction = CSYNC_INSTRUCTION_IGNORE;
     goto out;
   }
 
@@ -194,17 +206,17 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
    * does not change on rename.
    */
   if (csync_get_statedb_exists(ctx)) {
-    tmp = csync_statedb_get_stat_by_hash(ctx, h);
+    tmp = csync_statedb_get_stat_by_hash(ctx->statedb.db, h);
 
     if(tmp && tmp->phash == h ) { /* there is an entry in the database */
         /* we have an update! */
-        CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "Database entry found, compare: %" PRId64 " <-> %" PRId64 ", md5: %s <-> %s, inode: %" PRId64 " <-> %" PRId64,
-                  ((int64_t) fs->mtime), ((int64_t) tmp->modtime), fs->md5, tmp->md5, (int64_t) fs->inode, (int64_t) tmp->inode);
-        if( !fs->md5) {
+        CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "Database entry found, compare: %" PRId64 " <-> %" PRId64 ", etag: %s <-> %s, inode: %" PRId64 " <-> %" PRId64,
+                  ((int64_t) fs->mtime), ((int64_t) tmp->modtime), fs->etag, tmp->etag, (uint64_t) fs->inode, (uint64_t) tmp->inode);
+        if( !fs->etag) {
             st->instruction = CSYNC_INSTRUCTION_EVAL;
             goto out;
         }
-        if((ctx->current == REMOTE_REPLICA && !c_streq(fs->md5, tmp->md5 ))
+        if((ctx->current == REMOTE_REPLICA && !c_streq(fs->etag, tmp->etag ))
             || (ctx->current == LOCAL_REPLICA && (fs->mtime != tmp->modtime
 #if 0
                                                   || fs->inode != tmp->inode
@@ -224,15 +236,35 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
             st->instruction = CSYNC_INSTRUCTION_EVAL;
             goto out;
         }
+        if (type == CSYNC_FTW_TYPE_DIR && ctx->current == REMOTE_REPLICA
+                && c_streq(fs->file_id, tmp->file_id)) {
+            /* If both etag and file id are equal for a directory, read all contents from
+             * the database. */
+            CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "Reading from database: %s", path);
+            ctx->remote.read_from_db = true;
+        }
         st->instruction = CSYNC_INSTRUCTION_NONE;
     } else {
+        enum csync_vio_file_type_e tmp_vio_type = CSYNC_VIO_FILE_TYPE_UNKNOWN;
+
         /* check if it's a file and has been renamed */
         if (ctx->current == LOCAL_REPLICA) {
-            tmp = csync_statedb_get_stat_by_inode(ctx, fs->inode);
-            if (tmp && tmp->inode == fs->inode && (tmp->modtime == fs->mtime || fs->type == CSYNC_VIO_FILE_TYPE_DIRECTORY)) {
-                CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "inodes: %" PRId64 " <-> %" PRId64, (int64_t) tmp->inode, (int64_t) fs->inode);
+            tmp = csync_statedb_get_stat_by_inode(ctx->statedb.db, fs->inode);
+
+            /* translate the file type between the two stat types csync has. */
+            if( tmp && tmp->type == 0 ) {
+                tmp_vio_type = CSYNC_VIO_FILE_TYPE_REGULAR;
+            } else if( tmp && tmp->type == 2 ) {
+                tmp_vio_type = CSYNC_VIO_FILE_TYPE_DIRECTORY;
+            } else {
+                tmp_vio_type = CSYNC_VIO_FILE_TYPE_UNKNOWN;
+            }
+
+            if (tmp && tmp->inode == fs->inode && tmp_vio_type == fs->type
+                    && (tmp->modtime == fs->mtime || fs->type == CSYNC_VIO_FILE_TYPE_DIRECTORY)) {
+                CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "inodes: %" PRId64 " <-> %" PRId64, (uint64_t) tmp->inode, (uint64_t) fs->inode);
                 /* inode found so the file has been renamed */
-                st->instruction = CSYNC_INSTRUCTION_RENAME;
+                st->instruction = CSYNC_INSTRUCTION_EVAL_RENAME;
                 if (fs->type == CSYNC_VIO_FILE_TYPE_DIRECTORY) {
                     csync_rename_record(ctx, tmp->path, path);
                 }
@@ -242,9 +274,34 @@ static int _csync_detect_update(CSYNC *ctx, const char *file,
                 st->instruction = CSYNC_INSTRUCTION_NEW;
                 goto out;
             }
+        } else {
+            /* Remote Replica Rename check */
+            tmp = csync_statedb_get_stat_by_file_id(ctx->statedb.db, fs->file_id);
+            if(tmp ) {                           /* tmp existing at all */
+                if ((tmp->type == CSYNC_FTW_TYPE_DIR && fs->type != CSYNC_VIO_FILE_TYPE_DIRECTORY) ||
+                        (tmp->type == CSYNC_FTW_TYPE_FILE && fs->type != CSYNC_VIO_FILE_TYPE_REGULAR)) {
+                    CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "WARN: file types different is not!");
+                    st->instruction = CSYNC_INSTRUCTION_NEW;
+                    goto out;
+                }
+                st->instruction = CSYNC_INSTRUCTION_EVAL_RENAME;
+                if (fs->type == CSYNC_VIO_FILE_TYPE_DIRECTORY) {
+                    csync_rename_record(ctx, tmp->path, path);
+                } else {
+                    if( !c_streq(tmp->etag, fs->etag) ) {
+                        /* CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "ETags are different!"); */
+                        /* File with different etag, don't do a rename, but download the file again */
+                        st->instruction = CSYNC_INSTRUCTION_NEW;
+                    }
+                }
+                goto out;
+
+            } else {
+                /* file not found in statedb */
+                st->instruction = CSYNC_INSTRUCTION_NEW;
+                goto out;
+            }
         }
-        /* directory, remote and file not found in statedb */
-        st->instruction = CSYNC_INSTRUCTION_NEW;
     }
   } else  {
       st->instruction = CSYNC_INSTRUCTION_NEW;
@@ -255,9 +312,9 @@ out:
   /* Set the ignored error string. */
   if (st->instruction == CSYNC_INSTRUCTION_IGNORE) {
     if (excluded == CSYNC_FILE_EXCLUDE_LIST) {
-      st->error_string = c_strdup("File listed on ignore list.");
+      st->error_status = CSYNC_STATUS_INDIVIDUAL_IGNORE_LIST; /* File listed on ignore list. */
     } else if (excluded == CSYNC_FILE_EXCLUDE_INVALID_CHAR) {
-      st->error_string = c_strdup("File contains invalid characters.");
+      st->error_status = CSYNC_STATUS_INDIVIDUAL_IS_INVALID_CHARS;  /* File contains invalid characters. */
     }
   }
   if (st->instruction != CSYNC_INSTRUCTION_NONE && st->instruction != CSYNC_INSTRUCTION_IGNORE
@@ -275,10 +332,11 @@ out:
   st->gid   = fs->gid;
   st->nlink = fs->nlink;
   st->type  = type;
-  st->md5   = NULL;
-  if( fs->md5 ) {
-      st->md5  = c_strdup(fs->md5);
+  st->etag   = NULL;
+  if( fs->etag ) {
+      st->etag  = c_strdup(fs->etag);
   }
+  csync_vio_set_file_id(st->file_id, fs->file_id);
 
 fastout:  /* target if the file information is read from database into st */
   st->phash = h;
@@ -289,12 +347,14 @@ fastout:  /* target if the file information is read from database into st */
     case LOCAL_REPLICA:
       if (c_rbtree_insert(ctx->local.tree, (void *) st) < 0) {
         SAFE_FREE(st);
+        ctx->status_code = CSYNC_STATUS_TREE_ERROR;
         return -1;
       }
       break;
     case REMOTE_REPLICA:
       if (c_rbtree_insert(ctx->remote.tree, (void *) st) < 0) {
         SAFE_FREE(st);
+        ctx->status_code = CSYNC_STATUS_TREE_ERROR;
         return -1;
       }
       break;
@@ -316,7 +376,7 @@ int csync_walker(CSYNC *ctx, const char *file, const csync_vio_file_stat_t *fs,
 
   if (ctx->abort) {
     CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "Aborted!");
-    ctx->error_code = CSYNC_ERR_ABORTED;
+    ctx->status_code = CSYNC_STATUS_ABORTED;
     return -1;
   }
 
@@ -336,7 +396,7 @@ int csync_walker(CSYNC *ctx, const char *file, const csync_vio_file_stat_t *fs,
     if( h == 0 ) {
       return 0;
     }
-    st = csync_statedb_get_stat_by_hash(ctx, h);
+    st = csync_statedb_get_stat_by_hash(ctx->statedb.db, h);
     if( !st ) {
       return 0;
     }
@@ -362,62 +422,6 @@ int csync_walker(CSYNC *ctx, const char *file, const csync_vio_file_stat_t *fs,
   return rc;
 }
 
-/* check if the dirent entries for the directory can be read from db
- * instead really calling readdir which is costly over net.
- * For that, a single HEAD request is done on the directory to get its
- * id. If the ID has not changed remotely, this subtree hasn't changed
- * and can be read from db.
- */
-static int _check_read_from_db(CSYNC *ctx, const char *uri) {
-    int len;
-    uint64_t h;
-    csync_vio_file_stat_t *fs = NULL;
-    const char *md5_local  = NULL;
-    const char *md5_remote = NULL;
-    const char *mpath;
-    int rc = 0; /* FIXME: Error handling! */
-    csync_file_stat_t* tmp = NULL;
-
-    if( !c_streq( ctx->remote.uri, uri )) {
-        /* FIXME: The top uri can not be checked because there is no db entry for it */
-        if( strlen(uri) < strlen(ctx->remote.uri)+1 ) {
-            CSYNC_LOG(CSYNC_LOG_PRIORITY_ERROR, "check_read_from_db: uri is not a remote uri.");
-            /* FIXME: errno? */
-            return -1;
-        }
-        mpath = uri + strlen(ctx->remote.uri) + 1;
-        fs = csync_vio_file_stat_new();
-        if(fs == NULL) {
-            CSYNC_LOG(CSYNC_LOG_PRIORITY_ERROR, "check_read_from_db: memory fault.");
-            errno = ENOMEM;
-            return -1;
-        }
-        len = strlen( mpath );
-        h = c_jhash64((uint8_t *) mpath, len, 0);
-
-        /* search that folder in the db and check that the hash is the md5 (etag) is still the same */
-        if( csync_get_statedb_exists(ctx) ) {
-          tmp = csync_statedb_get_stat_by_hash(ctx, h);
-          if (tmp) {
-            md5_local = tmp->md5;
-            md5_remote = csync_vio_file_id(ctx, uri);
-
-            CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "Compare directory ids for %s: %s -> %s", mpath, md5_local, md5_remote );
-
-            if( c_streq(md5_local, md5_remote) ) {
-              ctx->remote.read_from_db = 1;
-            }
-            SAFE_FREE(md5_remote);
-            SAFE_FREE(md5_local);
-            SAFE_FREE(tmp);
-          }
-        }
-
-        csync_vio_file_stat_destroy(fs);
-    }
-    return rc;
-}
-
 /* File tree walker */
 int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
     unsigned int depth) {
@@ -436,47 +440,40 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
 
   if (uri[0] == '\0') {
     errno = ENOENT;
+    ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
     goto error;
   }
 
-  /* If remote, compare the id with the local id. If equal, read all contents from
-   * the database. */
   read_from_db = ctx->remote.read_from_db;
-  CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "  => Starting to ftw %s, read_from_db-Flag for: %d",
-          uri, read_from_db );
-  if( ctx->current == REMOTE_REPLICA && !do_read_from_db ) {
-      _check_read_from_db(ctx, uri);
-      do_read_from_db = (ctx->current == REMOTE_REPLICA && ctx->remote.read_from_db);
-      CSYNC_LOG(CSYNC_LOG_PRIORITY_TRACE, "Checking for read from db for %s: %d",
-                uri, ctx->remote.read_from_db );
-
-  }
 
   if ((dh = csync_vio_opendir(ctx, uri)) == NULL) {
     /* permission denied */
+    ctx->status_code = csync_errno_to_status(errno, CSYNC_STATUS_OPENDIR_ERROR);
     if (errno == EACCES) {
        return 0;
     } else if(errno == EIO ) {
       /* Proxy problems (ownCloud) */
-      ctx->error_code = CSYNC_ERR_PROXY;
-      goto error;
+      ctx->status_code = CSYNC_STATUS_PROXY_ERROR;
+    } else if(errno == ENOENT) {
+        asprintf( &ctx->error_string, "%s", uri);
     } else {
       C_STRERROR(errno, errbuf, sizeof(errbuf));
       CSYNC_LOG(CSYNC_LOG_PRIORITY_ERROR,
           "opendir failed for %s - %s (errno %d)",
           uri, errbuf, errno);
-      ctx->error_code = csync_errno_to_csync_error( CSYNC_ERR_UPDATE );
-      goto error;
     }
+    goto error;
   }
 
   while ((dirent = csync_vio_readdir(ctx, dh))) {
     const char *path = NULL;
+    size_t ulen = 0;
+    int flen;
     int flag;
 
     d_name = dirent->name;
     if (d_name == NULL) {
-      ctx->error_code = CSYNC_ERR_PARAM;
+      ctx->status_code = CSYNC_STATUS_READDIR_ERROR;
       goto error;
     }
 
@@ -488,31 +485,44 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
       continue;
     }
 
-    if (asprintf(&filename, "%s/%s", uri, d_name) < 0) {
+    flen = asprintf(&filename, "%s/%s", uri, d_name);
+    if (flen < 0) {
       csync_vio_file_stat_destroy(dirent);
       dirent = NULL;
-      ctx->error_code = CSYNC_ERR_PARAM;
+      ctx->status_code = CSYNC_STATUS_MEMORY_ERROR;
       goto error;
     }
 
     /* Create relative path */
     switch (ctx->current) {
       case LOCAL_REPLICA:
-        path = filename + strlen(ctx->local.uri) + 1;
+        ulen = strlen(ctx->local.uri) + 1;
         break;
       case REMOTE_REPLICA:
-        path = filename + strlen(ctx->remote.uri) + 1;
+        ulen = strlen(ctx->remote.uri) + 1;
         break;
       default:
         break;
     }
 
-    /* skip ".csync_journal.db" and ".csync_journal.db.ctmp" */
-    if (c_streq(path, ".csync_journal.db") || c_streq(path, ".csync_journal.db.ctmp")) {
+    if (((size_t)flen) < ulen) {
       csync_vio_file_stat_destroy(dirent);
       dirent = NULL;
-      SAFE_FREE(filename);
-      continue;
+      ctx->status_code = CSYNC_STATUS_UNSUCCESSFUL;
+      goto error;
+    }
+
+    path = filename + ulen;
+
+    /* skip ".csync_journal.db" and ".csync_journal.db.ctmp" */
+    if (c_streq(path, ".csync_journal.db")
+            || c_streq(path, ".csync_journal.db.ctmp")
+            || c_streq(path, ".csync_journal.db.ctmp-journal")
+            || c_streq(path, ".csync-progressdatabase")) {
+        csync_vio_file_stat_destroy(dirent);
+        dirent = NULL;
+        SAFE_FREE(filename);
+        continue;
     }
 
     /* == see if really stat has to be called. */
@@ -549,19 +559,19 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
     }
 
     if( ctx->current == LOCAL_REPLICA ) {
-        char *md5 = NULL;
+        char *etag = NULL;
         int len = strlen( path );
         uint64_t h = c_jhash64((uint8_t *) path, len, 0);
-        md5 = csync_statedb_get_uniqId( ctx, h, fs );
-        if( md5 ) {
-            SAFE_FREE(fs->md5);
-            fs->md5 = md5;
-            fs->fields |= CSYNC_VIO_FILE_STAT_FIELDS_MD5;
+        etag = csync_statedb_get_uniqId( ctx, h, fs );
+        if( etag ) {
+            SAFE_FREE(fs->etag);
+            fs->etag = etag;
+            fs->fields |= CSYNC_VIO_FILE_STAT_FIELDS_ETAG;
         }
-        if( c_streq(md5, "")) {
+        if( c_streq(etag, "")) {
           CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "Uniq ID from Database is EMPTY: %s", path);
         } else {
-          CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "Uniq ID from Database: %s -> %s", path, fs->md5 ? fs->md5 : "<NULL>" );
+          CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "Uniq ID from Database: %s -> %s", path, fs->etag ? fs->etag : "<NULL>" );
         }
     }
 
@@ -569,6 +579,7 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
 
     /* Call walker function for each file */
     rc = fn(ctx, filename, fs, flag);
+    /* this function may update ctx->current and ctx->read_from_db */
 
     if (ctx->current_fs && previous_fs && ctx->current_fs->child_modified) {
         previous_fs->child_modified = ctx->current_fs->child_modified;
@@ -577,16 +588,21 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
     if( ! do_read_from_db ) {
         csync_vio_file_stat_destroy(fs);
     } else {
-        SAFE_FREE(fs->md5);
+        SAFE_FREE(fs->etag);
     }
 
     if (rc < 0) {
+      if (CSYNC_STATUS_IS_OK(ctx->status_code)) {
+          ctx->status_code = CSYNC_STATUS_UPDATE_ERROR;
+      }
+
       csync_vio_closedir(ctx, dh);
       ctx->current_fs = previous_fs;
       goto done;
     }
 
-    if (flag == CSYNC_FTW_FLAG_DIR && depth) {
+    if (flag == CSYNC_FTW_FLAG_DIR && depth
+        && (!ctx->current_fs || ctx->current_fs->instruction != CSYNC_INSTRUCTION_IGNORE)) {
       rc = csync_ftw(ctx, filename, fn, depth - 1);
       if (rc < 0) {
         ctx->current_fs = previous_fs;
@@ -597,10 +613,17 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
       if (ctx->current_fs && !ctx->current_fs->child_modified
           && ctx->current_fs->instruction == CSYNC_INSTRUCTION_EVAL) {
         ctx->current_fs->instruction = CSYNC_INSTRUCTION_NONE;
-        ctx->current_fs->should_update_md5 = true;
+        ctx->current_fs->should_update_etag = true;
       }
     }
+
+    if (ctx->current_fs && (ctx->current_fs->instruction == CSYNC_INSTRUCTION_EVAL ||
+        ctx->current_fs->instruction == CSYNC_INSTRUCTION_NEW)) {
+        ctx->current_fs->should_update_etag = true;
+    }
+
     ctx->current_fs = previous_fs;
+    ctx->remote.read_from_db = read_from_db;
     SAFE_FREE(filename);
     csync_vio_file_stat_destroy(dirent);
     dirent = NULL;
@@ -615,11 +638,10 @@ done:
   SAFE_FREE(filename);
   return rc;
 error:
-  ctx->remote.read_from_db = read_from_db;
-
   if (dh != NULL) {
     csync_vio_closedir(ctx, dh);
   }
+  ctx->remote.read_from_db = read_from_db;
   SAFE_FREE(filename);
   return -1;
 }
