@@ -3,19 +3,19 @@
  *
  * Copyright (c) 2012 Klaas Freitag <freitag@owncloud.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation version 2 of
- * of the License.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <string.h>
@@ -42,7 +42,7 @@
 #define DEBUG_HBF(...) { if(transfer->log_cb) { \
         char buf[1024];                         \
         snprintf(buf, 1024, __VA_ARGS__);       \
-        transfer->log_cb(__FUNCTION__, buf);    \
+        transfer->log_cb(__FUNCTION__, buf, transfer->user_data);    \
   }  }
 
 // #endif
@@ -95,6 +95,7 @@ hbf_transfer_t *hbf_init_transfer( const char *dest_uri ) {
     transfer->start_id = 0;
     transfer->block_size = DEFAULT_BLOCK_SIZE;
     transfer->threshold = transfer->block_size;
+    transfer->modtime_accepted = 0;
 
     return transfer;
 }
@@ -124,11 +125,12 @@ Hbf_State hbf_splitlist(hbf_transfer_t *transfer, int fd ) {
   transfer->fd        = fd;
   transfer->stat_size = sb.st_size;
   transfer->modtime   = sb.st_mtime;
+  transfer->previous_etag = NULL;
 #ifndef NDEBUG
   transfer->calc_size = 0;
 #endif
 
-  DEBUG_HBF("hbf_splitlist: block_size: %" PRId64 " threshold: %" PRId64 " st_size: %" PRId64 "\n", transfer->block_size, transfer->threshold, sb.st_size );
+  DEBUG_HBF("block_size: %" PRId64 " threshold: %" PRId64 " st_size: %" PRId64, transfer->block_size, transfer->threshold, sb.st_size );
 
 
   /* calc the number of blocks to split in */
@@ -153,7 +155,7 @@ Hbf_State hbf_splitlist(hbf_transfer_t *transfer, int fd ) {
     blk_size   = 0;
   }
 
-  DEBUG_HBF("hbf_splitlist: num_blocks: %" PRId64 " rmainder: %" PRId64 " blk_size: %" PRId64 "\n", num_blocks, remainder, blk_size );
+  DEBUG_HBF("num_blocks: %" PRId64 " rmainder: %" PRId64 " blk_size: %" PRId64, num_blocks, remainder, blk_size );
 
 
   if( num_blocks ) {
@@ -185,7 +187,7 @@ Hbf_State hbf_splitlist(hbf_transfer_t *transfer, int fd ) {
           /* store the block data into the result array in the transfer */
           *((transfer->block_arr)+cnt) = block;
 
-          DEBUG_HBF("hbf_splitlist: created block %d   (start: %" PRId64 "  size: %" PRId64 ")", cnt, block->start, block->size );
+          DEBUG_HBF("created block %d   (start: %" PRId64 "  size: %" PRId64 ")", cnt, block->start, block->size );
       }
 
 #ifndef NDEBUG
@@ -208,7 +210,7 @@ void hbf_free_transfer( hbf_transfer_t *transfer ) {
     }
     free( transfer->block_arr );
     free( transfer->url );
-
+    free( transfer->file_id );
     if( transfer->error_string) free( (void*) transfer->error_string );
 
     free( transfer );
@@ -286,7 +288,7 @@ static int _hbf_dav_request(hbf_transfer_t *transfer, ne_request *req, int fd, h
     if( ! (blk && req) ) return HBF_PARAM_FAIL;
 
     ne_set_request_body_fd(req, fd, blk->start, blk->size);
-    DEBUG_HBF("HBF: Block: %d , Start: %" PRId64 " and Size: %" PRId64 "", blk->seq_number, blk->start, blk->size );
+    DEBUG_HBF("Block: %d , Start: %" PRId64 " and Size: %" PRId64 "", blk->seq_number, blk->start, blk->size );
     res = ne_request_dispatch(req);
 
     req_status = ne_get_status( req );
@@ -315,6 +317,18 @@ static int _hbf_dav_request(hbf_transfer_t *transfer, ne_request *req, int fd, h
             } else {
                 /* DEBUG_HBF("OOOOOOOO No etag returned!"); */
             }
+
+            /* check if the server was able to set the mtime already. */
+            etag = ne_get_response_header(req, "X-OC-MTime");
+            if( etag && strcmp(etag, "accepted") == 0 ) {
+                /* the server acknowledged that the mtime was set. */
+                transfer->modtime_accepted = 1;
+            }
+
+            etag = ne_get_response_header(req, "OC-FileID");
+            if( etag ) {
+                transfer->file_id = strdup( etag );
+            }
         }
         break;
     case NE_AUTH:
@@ -347,7 +361,7 @@ static int _hbf_dav_request(hbf_transfer_t *transfer, ne_request *req, int fd, h
     return state;
 }
 
-static Hbf_State validate_source_file( hbf_transfer_t *transfer ) {
+Hbf_State hbf_validate_source_file( hbf_transfer_t *transfer ) {
   Hbf_State state = HBF_SUCCESS;
   hbf_stat_t sb;
 
@@ -396,8 +410,10 @@ static Hbf_State _hbf_transfer_no_chunk(ne_session *session, hbf_transfer_t *tra
     if (!req)
         return HBF_MEMORY_FAIL;
 
+    ne_add_request_header( req, "Content-Type", "application/octet-stream");
+
     ne_set_request_body_fd(req, transfer->fd, 0, transfer->stat_size);
-    DEBUG_HBF("HBF: chunking not supported for %s\n", transfer->url);
+    DEBUG_HBF("HBF: chunking not supported for %s", transfer->url);
     res = ne_request_dispatch(req);
     req_status = ne_get_status( req );
 
@@ -427,6 +443,10 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
         state = HBF_PARAM_FAIL;
     }
 
+    if(state == HBF_TRANSFER_SUCCESS) {
+        DEBUG_HBF("%s request to %s", verb, transfer->url);
+    }
+
     for( cnt=0; state == HBF_TRANSFER_SUCCESS && cnt < transfer->block_cnt; cnt++ ) {
         /* cnt goes from O to block_cnt,  but block_id starts at start_id and wrap around
          * That way if we have not finished uploaded when we reach block_cnt, we re-upload
@@ -439,7 +459,7 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
         if( ! block ) state = HBF_PARAM_FAIL;
 
         if( transfer->abort_cb ) {
-            int do_abort = (transfer->abort_cb)();
+            int do_abort = (transfer->abort_cb)(transfer->user_data);
             if( do_abort ) {
               state = HBF_USER_ABORTED;
               transfer->start_id = block_id  % transfer->block_cnt;
@@ -456,7 +476,7 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
         if( state == HBF_TRANSFER_SUCCESS ) {
           if( transfer->block_cnt > 1 && cnt > 0 ) {
             /* The block count is > 1, check size and mtime before transmitting. */
-            state = validate_source_file(transfer);
+            state = hbf_validate_source_file(transfer);
             if( state == HBF_SOURCE_FILE_CHANGE ) {
               /* The source file has changed meanwhile */
             }
@@ -468,11 +488,23 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
 
             if( req ) {
                 char buf[21];
+
                 snprintf(buf, sizeof(buf), "%"PRId64, transfer->stat_size);
                 ne_add_request_header(req, "OC-Total-Length", buf);
+                if( transfer->modtime > 0 ) {
+                    snprintf(buf, sizeof(buf), "%"PRId64, transfer->modtime);
+                    ne_add_request_header(req, "X-OC-Mtime", buf);
+                }
+
+                if( transfer->previous_etag ) {
+                  ne_add_request_header(req, "If-Match", transfer->previous_etag);
+                }
+
                 if( transfer->block_cnt > 1 ) {
                   ne_add_request_header(req, "OC-Chunked", "1");
                 }
+                ne_add_request_header( req, "Content-Type", "application/octet-stream");
+
                 state = _hbf_dav_request(transfer,  req, transfer->fd, block );
 
                 if( state != HBF_TRANSFER_SUCCESS && state != HBF_SUCCESS) {
@@ -497,6 +529,10 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
                     }
                 }
 
+                if (state == HBF_TRANSFER_SUCCESS && transfer->chunk_finished_cb) {
+                    transfer->chunk_finished_cb(transfer, block_id, transfer->user_data);
+                }
+
             } else {
                 state = HBF_MEMORY_FAIL;
             }
@@ -505,9 +541,7 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
     }
 
     /* do the source file validation finally (again). */
-    if( state == HBF_SUCCESS ) {
-        state = validate_source_file(transfer);
-    } else if( state == HBF_TRANSFER_SUCCESS ) {
+    if( state == HBF_TRANSFER_SUCCESS ) {
         /* This means that no etag was returned on one of the chunks to indicate
          * that the upload was finished. */
         state = HBF_TRANSFER_NOT_ACKED;
@@ -533,6 +567,37 @@ int hbf_fail_http_code( hbf_transfer_t *transfer )
   return 200;
 }
 
+const char *hbf_transfer_etag( hbf_transfer_t *transfer )
+{
+    int cnt;
+    const char *etag = NULL;
+
+    if( ! transfer ) return 0;
+
+    /* Loop over all parts and do a assertion that there is only one etag. */
+    for( cnt = 0; cnt < transfer->block_cnt; cnt++ ) {
+        int block_id = (cnt + transfer->start_id) % transfer->block_cnt;
+        hbf_block_t *block = transfer->block_arr[block_id];
+        if( block->etag ) {
+            if( etag && strcmp(etag, block->etag) != 0 ) {
+                /* multiple etags in the transfer, not equal. */
+                DEBUG_HBF( "WARN: etags are not equal in blocks of one single transfer." );
+            }
+            etag = block->etag;
+        }
+    }
+    return etag;
+}
+
+const char *hbf_transfer_file_id( hbf_transfer_t *transfer )
+{
+    const char *re = NULL;
+    if(transfer) {
+        re = transfer->file_id;
+    }
+    return re;
+}
+
 const char *hbf_error_string(hbf_transfer_t *transfer, Hbf_State state)
 {
     const char *re;
