@@ -3,22 +3,20 @@
  *
  * Copyright (c) 2012 by Klaas Freitag <freitag@owncloud.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * vim: ts=2 sw=2 et cindent
- **/
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #ifndef _HBF_SEND_H
 #define _HBF_SEND_H
@@ -74,13 +72,13 @@ struct hbf_block_s {
     int tries;
 };
 
-/* Callback for to check on abort */
-typedef int (*hbf_abort_callback) ();
-typedef void (*hbf_log_callback) (const char *, const char *);
-
-
 typedef struct hbf_transfer_s hbf_transfer_t;
 
+/* Callback for to check on abort */
+typedef int (*hbf_abort_callback) (void *);
+typedef void (*hbf_log_callback) (const char *, const char *, void*);
+typedef void (*hbf_chunk_finished_callback) (hbf_transfer_t*,int, void*);
+
 struct hbf_transfer_s {
     hbf_block_t **block_arr;
     int block_cnt;
@@ -97,8 +95,13 @@ struct hbf_transfer_s {
     int64_t block_size;
     int64_t threshold;
 
+    void *user_data;
     hbf_abort_callback abort_cb;
     hbf_log_callback log_cb;
+    hbf_chunk_finished_callback chunk_finished_cb;
+    int modtime_accepted;
+    const char *previous_etag; /* etag send as the If-Match http header */
+    char *file_id;
 
 #ifndef NDEBUG
     int64_t calc_size;
@@ -115,6 +118,10 @@ void hbf_free_transfer( hbf_transfer_t *transfer );
 
 const char *hbf_error_string(hbf_transfer_t* transfer, Hbf_State state);
 
+const char *hbf_transfer_etag( hbf_transfer_t *transfer );
+
+const char *hbf_transfer_file_id( hbf_transfer_t *transfer );
+
 void hbf_set_abort_callback( hbf_transfer_t *transfer, hbf_abort_callback cb);
 void hbf_set_log_callback( hbf_transfer_t *transfer, hbf_log_callback cb);
 
@@ -124,6 +131,8 @@ void hbf_set_log_callback( hbf_transfer_t *transfer, hbf_log_callback cb);
  */
 int hbf_fail_http_code( hbf_transfer_t *transfer );
 
+Hbf_State hbf_validate_source_file( hbf_transfer_t *transfer );
+
 #ifdef __cplusplus
 }
 #endif
