@@ -1,23 +1,22 @@
 /*
  * cynapses libc functions
  *
- * Copyright (c) 2008 by Andreas Schneider <mail@cynapses.org>
+ * Copyright (c) 2008-2013 by Andreas Schneider <asn@cryptomilk.org>
+ * Copyright (c) 2012-2013 by Klaas Freitag <freitag@owncloud.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * vim: ts=2 sw=2 et cindent
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /**
@@ -152,45 +151,78 @@ char *c_uppercase(const char* str);
 char *c_lowercase(const char* str);
 
 /**
- * @brief Convert a multibyte string to utf8 (Win32).
+ * @brief Convert a platform locale string to utf8.
+ *
+ * This function is part of the multi platform abstraction of basic file
+ * operations to handle various platform encoding correctly.
+ *
+ * Instead of using the standard file operations the multi platform aliases
+ * defined in c_private.h have to be used instead.
+ *
+ * To convert path names returned by these functions to the internally used
+ * utf8 format this function has to be used. The returned string has to
+ * be freed by c_free_locale_string(). On some platforms this method allocates
+ * memory and on others not but it has never to be cared about.
  *
  * @param  str     The multibyte encoded string to convert
  *
  * @return The malloced converted string or NULL on error.
+ *
+ * @see c_free_locale_string()
+ * @see c_utf8_to_locale()
+ *
  */
- char*   c_utf8(const _TCHAR *str);
+ char*   c_utf8_from_locale(const mbchar_t *str);
 
 /**
- * @brief Convert a utf8 encoded string to multibyte (Win32).
+ * @brief Convert a utf8 encoded string to platform specific locale.
+ *
+ * This function is part of the multi platform abstraction of basic file
+ * operations to handle various platform encoding correctly.
+ *
+ * Instead of using the standard file operations the multi platform aliases
+ * defined in c_private.h have to be used instead.
+ *
+ * To convert path names as input for the cross platform functions from the
+ * internally used utf8 format, this function has to be used.
+ * The returned string has to be freed by c_free_locale_string(). On some
+ * platforms this method allocates memory and on others not but it has never
+ * sto be cared about.
  *
  * @param  str     The utf8 string to convert.
  *
  * @return The malloced converted multibyte string or NULL on error.
+ *
+ * @see c_free_locale_string()
+ * @see c_utf8_from_locale()
+ *
  */
-_TCHAR* c_multibyte(const char *wstr);
+mbchar_t* c_utf8_to_locale(const char *wstr);
 
 #if defined(_WIN32) || defined(WITH_ICONV)
+
 /**
- * @brief Free buffer malloced by c_multibyte.
+ * @brief Free buffer malloced by c_utf8_from_locale or c_utf8_to_locale().
  *
- * @param  buf     The buffer to free.
- */
-
-#define c_free_multibyte(x) SAFE_FREE(x)
-
-/**
- * @brief Free buffer malloced by c_utf8.
+ * This function is part of the multi platform abstraction of basic file
+ * operations to handle various platform encoding correctly.
+ *
+ * Instead of using the standard file operations the multi platform aliases
+ * defined in c_private.h have to be used instead.
+ *
+ * This function frees the memory that was allocated by a previous call to
+ * c_utf8_to_locale() or c_utf8_from_locale().
  *
  * @param  buf     The buffer to free.
  *
+ * @see c_utf8_from_locale(), c_utf8_to_locale()
+ *
  */
-#define c_free_utf8(x) SAFE_FREE(x)
+#define c_free_locale_string(x) SAFE_FREE(x)
 #else
-#define c_free_multibyte(x) (void)x
-#define c_free_utf8(x) (void)x
+#define c_free_locale_string(x) (void)x
 #endif
 
-
 /**
  * }@
  */
