@@ -3,6 +3,7 @@
 #include "torture.h"
 
 #define CSYNC_TEST 1
+#include "csync.h"
 #include "csync_config.c"
 
 #define TESTCONF "/tmp/check_csync1/csync.conf"
@@ -41,10 +42,11 @@ static void teardown(void **state) {
 
 static void check_csync_config_copy_default(void **state)
 {
-    CSYNC *csync = *state;
     int rc;
 
-    rc = _csync_config_copy_default(csync, TESTCONF);
+    (void) state; /* unused */
+
+    rc = _csync_config_copy_default(TESTCONF);
     assert_int_equal(rc, 0);
 }
 
@@ -53,8 +55,15 @@ static void check_csync_config_load(void **state)
     CSYNC *csync = *state;
     int rc;
 
-    rc = csync_config_load(csync, TESTCONF);
+    rc = _csync_config_copy_default(TESTCONF);
     assert_int_equal(rc, 0);
+
+    rc = csync_config_parse_file(csync, TESTCONF);
+    assert_int_equal(rc, 0);
+
+    assert_false(csync->options.with_conflict_copys);
+    assert_int_equal(csync->options.max_time_difference, 10);
+    assert_int_equal(csync->options.max_depth, 50);
 }
 
 int torture_run_tests(void)
