@@ -1,19 +1,19 @@
 #
 # Copyright (c) 2013 Klaas Freitag <freitag@owncloud.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2.1 of the License, or (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program (see the file COPYING); if not, write to the
-# Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 ################################################################
 # Contributors:
@@ -30,27 +30,42 @@ use File::Glob ':glob';
 use Carp::Assert;
 use Digest::MD5;
 use Unicode::Normalize;
+use LWP::UserAgent;
+use HTTP::Request::Common;
+use File::Basename;
+
 use Encode qw(from_to);
 use utf8;
 if ($^O eq "darwin") {
   eval "require Encode::UTF8Mac";
 }
 
+use open ':encoding(utf8)';
+
 use vars qw( @ISA @EXPORT @EXPORT_OK $d %config);
 
 our $owncloud   = "http://localhost/oc/remote.php/webdav/";
 our $user       = "joe";
 our $passwd     = 'XXXXX'; # Mind to be secure.
-our $ld_libpath = "/home/kf/owncloud.com/buildcsync/modules";
-our $csync      = "/home/kf/owncloud.com/buildcsync/client/ocsync";
+our $ld_libpath = "/home/joe/owncloud.com/buildcsync/modules";
+our $csync      = "/home/joe/owncloud.com/buildcsync/client/ocsync";
 our $remoteDir;
-our $localDir   = "turbo";
+our $localDir;
+our $infoCnt = 1;
 
 
 @ISA        = qw(Exporter);
-@EXPORT     = qw( initTesting createRemoteDir createLocalDir cleanup csync assertLocalDirs assertLocalAndRemoteDir 
-                  glob_put put_to_dir localDir remoteDir localCleanup createLocalFile remoteCleanup);
+@EXPORT     = qw( initTesting createRemoteDir createLocalDir cleanup csync
+                  assertLocalDirs assertLocalAndRemoteDir glob_put put_to_dir 
+                  putToDirLWP localDir remoteDir localCleanup createLocalFile md5OfFile
+                  remoteCleanup server initLocalDir initRemoteDir moveRemoteFile
+                  printInfo remoteFileId);
 
+sub server
+{
+  return $owncloud;
+}
+                  
 sub fromFileName($)
 {
   my ($file) = @_;
@@ -63,18 +78,20 @@ sub fromFileName($)
 }
 
 
-sub initTesting()
+sub initTesting(;$)
 {
+  my ($prefix) = @_;
+  
   if( -r "./t1.cfg" ) {
     my %config = do 't1.cfg';
     warn "Could not parse t1.cfg: $!\n" unless %config;
     warn "Could not do t1.cfg: $@\n" if $@;
 
-    $user   = $config{user} if( $config{user} );
-    $passwd = $config{passwd} if( $config{passwd} );
-    $owncloud = $config{url}  if( $config{url} );
+    $user       = $config{user} if( $config{user} );
+    $passwd     = $config{passwd} if( $config{passwd} );
+    $owncloud   = $config{url}  if( $config{url} );
     $ld_libpath = $config{ld_libpath} if( $config{ld_libpath} );
-    $csync   = $config{csync} if( $config{csync} );
+    $csync      = $config{csync} if( $config{csync} );
     print "Read t1.cfg: $config{url}\n";
   }
 
@@ -88,21 +105,45 @@ sub initTesting()
 		  -user=> $user,
 		  -pass=> $passwd );
   # $d->DebugLevel(3);
-
-  $remoteDir = sprintf( "t1-%#.3o/", rand(1000) );
-  $localDir .= "/" unless( $localDir =~ /\/$/ );
+  $prefix = "t1" unless( defined $prefix );
   
-  print "Working in remote dir $remoteDir\n";
-  createLocalDir();
-  createRemoteDir( $remoteDir );
-  $owncloud .= $remoteDir;
-  }
+  my $dirId = sprintf("%#.3o", rand(1000));
+  my $dir = sprintf( "%s-%s/", $prefix, $dirId );
+  
+  $localDir = $dir;
+  $localDir .= "/" unless( $localDir =~ /\/$/ );
+  $remoteDir = $dir;
+  
+  initRemoteDir();
+  initLocalDir();
+  printf( "Test directory name is %s\n", $dir );
+}
 
-sub createRemoteDir($)
+# Call this first to create the unique test dir stored in
+# the global var $remoteDir;
+sub initRemoteDir
+{
+  $d->open( $owncloud );
+  $owncloud .= $remoteDir;
+  
+  my $re = $d->mkcol( $owncloud );
+  if( $re == 0 ) {
+    print "Failed to create test dir $owncloud\n";
+    exit 1;
+  }
+  # $owncloud .= $remoteDir;
+}
+
+sub initLocalDir
+{
+  mkdir ($localDir, 0777 );
+}
+
+sub createRemoteDir(;$)
 {
     my ($dir) = @_;
 
-    my $url = $owncloud . $dir ;
+    my $url = $owncloud . $dir;
 
     $d->open( $owncloud );
     print $d->message . "\n";
@@ -130,9 +171,9 @@ sub cleanup()
 {
   # ==================================================================
 
-  print "\n###########################################\n";
-  print "    all cool - tests succeeded in $remoteDir.\n";
-  print "###########################################\n";
+  print "\n################################################\n";
+  printf( "    all cool - %d tests succeeded in %s.\n", $infoCnt-1, $remoteDir);
+  print "#################################################\n";
 
   print "\nInterrupt before cleanup in 4 seconds...\n";
   sleep(4);
@@ -172,7 +213,7 @@ sub csync(  )
     $url = "owncloud://$user:$passwd@". $url;
     print "CSync URL: $url\n";
 
-    my $args = "--exclude-file=exclude.cfg -c";
+    my $args = ""; # "--exclude-file=exclude.cfg -c";
     my $cmd = "LD_LIBRARY_PATH=$ld_libpath $csync $args $localDir $url";
     print "Starting: $cmd\n";
 
@@ -193,7 +234,7 @@ sub assertLocalDirs( $$ )
         next if( -d "$dir1/$_"); # don't compare directory sizes.
 	my $s1 = -s "$dir1/$_";
 	my $s2 = -s "$dir2/$_";
-	assert( $s1 == $s2 );
+	assert( $s1 == $s2, "$dir1/$_ <-> $dir2/$_" );
     }
     closedir $dh;
 }
@@ -234,23 +275,21 @@ sub assertFile($$)
   my $remoteSize = $res->get_property( "getcontentlength" );
   if( $remoteSize ) { # directories do not have a contentlength
     print "Local versus Remote size: $localSize <-> $remoteSize\n";
-    assert( $localSize == $remoteSize, "File sizes differ!\n" );
+    assert( $localSize == $remoteSize, "File sizes differ" );
   }
 }
 
 sub registerSeen($$)
 {
   my ($seenRef, $file) = @_;
-
-  $file =~ s/t1-\d+\//t1\//;
   $seenRef->{$file} = 1;
 }
 
 sub traverse( $$ )
 {
     my ($remote, $acceptConflicts) = @_;
+    $remote .= '/' unless $remote =~ /(^|\/)$/;
     printf("===============> $remote\n");
-    $remote .= '/' unless $remote =~ /\/$/;
     
     my $url = $owncloud . $remote;
     my %seen;
@@ -272,7 +311,7 @@ sub traverse( $$ )
 		    print "Checking file: $remote$filename\n";
 		    my $localFile = $localDir . $remote . $filename;
 		    registerSeen( \%seen, $localFile );
-		    $localFile =~ s/t1-\d+\//t1\//;
+		    # $localFile =~ s/t1-\d+\//t1\//;
 
 		    assertFile( $localFile, $res );
 		}
@@ -283,9 +322,9 @@ sub traverse( $$ )
     }
 
     # Check the directory contents
-    my $localpath = localDir().$remote;
-    $localpath =~ s/t1-\d+\//t1\//;
-
+    my $localpath = localDir();
+    $localpath .= $remote if( $remote ne "/" );
+  print "#### localpath = " . $localpath . "\n";
     opendir(my $dh, $localpath ) || die;
     # print Dumper( %seen );
     while( readdir $dh ) {
@@ -305,7 +344,7 @@ sub traverse( $$ )
 
 	$isHere = 1 if( $acceptConflicts && !$isHere && $f =~ /_conflict/ );
 	$isHere = 1 if( $f =~ /\.csync/ );
-	assert( $isHere, "Filename only local, but not remote: $f" );
+	assert( $isHere, "Filename local, but not remote: $f" );
     }
 
     # Check if there was something remote that we havent locally.
@@ -328,7 +367,7 @@ sub glob_put( $$ )
 {
     my( $globber, $target ) = @_;
 
-    $target = $owncloud . $target;
+    # $target = $owncloud . $target;
     
     $d->open( $target );
 
@@ -341,11 +380,15 @@ sub glob_put( $$ )
 	    if( -d $lfile ) {
 	      $d->mkcol( $puturl );
 	    } else {
+	    $lfile = $llfile;
+	    $puturl = $target;
 	      print "   *** Putting $lfile to $puturl\n";
-
-	      if( ! $d->put( -local=>$lfile, -url=> $puturl ) ) {
-		print "   ### FAILED to put: ". $d->message . '\n';
-	      }
+	      # putToDirLWP( $lfile, $puturl );
+	      put_to_dir($lfile, $puturl);
+	      
+	      # if( ! $d->put( -local=>$lfile, -url=> $puturl ) ) {
+	      #print "   ### FAILED to put: ". $d->message . '\n';
+	      # s}
 	    }
 	}
 	  
@@ -368,14 +411,48 @@ sub put_to_dir( $$ )
     }
 }
 
+# The HTTP DAV module often does a PROPFIND before it really PUTs. That 
+# is not neccessary if we know that the directory is really there.
+# Use this function in this case:
+sub putToDirLWP($$)
+{
+    my ($file, $dir) = @_;
+
+    $dir .="/" unless $dir =~ /\/$/;
+
+    my $filename = $file;
+    my $basename = basename $filename;
+
+    $dir =~ s/^\.\///;
+    my $puturl = $owncloud . $dir. $basename;
+    # print "putToDir LWP puts $filename to $puturl\n";
+    die("Could not open $filename: $!") unless( open FILE, "$filename" );
+    binmode FILE,  ":utf8";;
+    my $string = <FILE>;
+    close FILE;
+
+    my $ua  = LWP::UserAgent->new();
+    $ua->agent( "ownCloudTest_$localDir");
+    my $req = PUT $puturl, Content_Type => 'application/octet-stream',
+		  Content => $string;
+    $req->authorization_basic($user, $passwd);
+    my $response = $ua->request($req);
+    
+    if ($response->is_success()) {
+      # print "OK: ", $response->content;
+    } else {
+      die( "HTTP PUT failed: " . $response->as_string );
+    }
+}
+
 sub createLocalFile( $$ ) 
 {
   my ($fname, $size) = @_;
   $size = 1024 unless( $size );
   
   my $md5 = Digest::MD5->new;
-   
-  open(FILE, ">", $localDir . $fname) or die "Can't open $fname for writing ($!)";
+
+  open(FILE, ">", $fname) or die "Can't open $fname for writing ($!)";
   
   my $minimum = 32;
   my $range = 96;
@@ -394,5 +471,69 @@ sub createLocalFile( $$ )
   return $md5->hexdigest; 
 }
 
-#
+sub md5OfFile( $ ) 
+{
+  my ($file) = @_;
+  
+  open FILE, "$file";
 
+  my $ctx = Digest::MD5->new;
+  $ctx->addfile (*FILE);
+  my $hash = $ctx->hexdigest;
+  close (FILE);
+  
+  return $hash;
+}
+
+sub moveRemoteFile($$)
+{
+  my ($from, $to) = @_;
+    
+  my $fromUrl = $owncloud . $from;
+  my $toUrl = $owncloud . $to;
+  
+  $d->move($fromUrl, $toUrl);
+  
+}
+
+sub printInfo($)
+{
+  my ($info) = @_;
+  my $tt = 6+length( $info );
+  
+  print "#" x $tt;
+  printf( "\n# %2d. %s", $infoCnt, $info );
+  print "\n" unless $info =~ /\n$/;
+  print "#" x $tt;
+  print "\n";
+  
+  $infoCnt++;
+}
+
+sub remoteFileId($$)
+{
+  my ($fromDir, $file) = @_;
+  my $fromUrl = $owncloud . $fromDir;
+  my $id;
+
+  if( my $r = $d->propfind( -url => $fromUrl, -depth => 1 ) ) {
+    if ( $r->is_collection ) {
+      # print "Collection\n";
+
+      foreach my $res ( $r->get_resourcelist->get_resources() ) {
+	my $filename = $res->get_property("rel_uri");
+	# print "OOOOOOOOOOOOOO $filename " . $res->get_property('id') . "\n";
+	if( $file eq $filename || $filename eq $file . "/" ) {
+	  $id = $res->get_property('id') || "";
+	}
+      }
+    } else {
+      # print "OOOOOOOOOOOOOOOOOOO " . $r->get_property("rel_uri");
+      $id = $r->get_property('id') || "";
+    }
+  }
+  print "## ID of $file: $id\n";
+  return $id;
+}
+
+#
